import os
import sys
import shutil
import pandas as pd
from thyrsis.log import logger
from thyrsis.database import sqlite
from thyrsis.settings import Settings

BDLHESDATA = os.path.join(os.getenv("HOME"), "git_bdlhes", "data")

def convert_radionuclide(s):
    if s in ["RDX", "tracer"]:
        return None
    else:
        return s


def convert_chemical_element(s):
    return s.split("-")[0]


def export_to_bdlhes(conn, bdlhes_path=BDLHESDATA):
    cur = conn.cursor()

    sites = {s[0]:s[1] for s in cur.execute("SELECT * from sites").fetchall()}
    points = cur.execute("SELECT OGC_FID, sid, nom from points_interet").fetchall()
    dfp = pd.DataFrame(points, columns=["pid", "sid", "nom"])

    elements = cur.execute("SELECT id, nom from elements_chimiques").fetchall()
    dfe = pd.DataFrame(elements, columns=["eid", "name"])

    concentrations = cur.execute("""SELECT pid, date, type, eid, concentration, incertitude_concentration
        FROM mesures WHERE concentration is not NULL""").fetchall()
    dfc = pd.DataFrame(concentrations, columns=["pid", "measure_time", "measure_legend", "eid", "measure_value", "measure_uncertainty"])

    potentiels = cur.execute("""SELECT pid, date, type, potentiel
        FROM mesures WHERE potentiel is not NULL""").fetchall()
    dfh = pd.DataFrame(potentiels, columns=["pid", "measure_time", "measure_legend", "groundwater_level"])

    for sid, site in sites.items():
        df = dfc[dfc["pid"].isin(dfp.loc[dfp.sid == sid, "pid"])].copy()
        if not df.empty:
            df["measure_unit"] = "kg/m³"
            df["sample_type"] = "eau souterraine"
            df["station_name"] = df.pid.replace(dfp.set_index("pid").to_dict()["nom"])
            df["chemical_element"] = df.eid.replace(dfe.set_index("eid").to_dict()["name"])
            df["radionuclide"] = df["chemical_element"].apply(convert_radionuclide)
            df["chemical_element"] = df["chemical_element"].apply(convert_chemical_element)
            df.drop(columns=["eid", "pid"], inplace=True)
            df = df.reindex(columns=["station_name", "chemical_element", "radionuclide",
                "measure_time", "measure_unit", "measure_value", "measure_uncertainty", "measure_legend", "sample_type"])
            out_dir = os.path.join(bdlhes_path, site, "RAW", "CHIMIE", "SYNTHESE")
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            fout = os.path.join(out_dir, "chemical_analysis_thyrsis.csv")
            logger.notice("Exporting chemical for site", site, "into file", fout)
            df.to_csv(fout, encoding="utf-8", index=False, header=True, sep=";")

        df = dfh[dfh["pid"].isin(dfp.loc[dfp.sid == sid, "pid"])].copy()
        if not df.empty:
            df["measure_unit"] = "m"
            df["station_name"] = df.pid.replace(dfp.set_index("pid").to_dict()["nom"])
            df.drop(columns=["pid"], inplace=True)
            df = df.reindex(columns=["station_name",
                "measure_time", "measure_unit", "groundwater_level", "measure_uncertainty", "measure_legend"])
            out_dir = os.path.join(bdlhes_path, site, "RAW", "HYDROGEOLOGIE", "SYNTHESE")
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            fout = os.path.join(out_dir, "hydrogeologie_thyrsis.csv")
            logger.notice("Exporting hydrogeology for site", site, "into file", fout)
            df.to_csv(fout, encoding="utf-8", index=False, header=True, sep=";")

if __name__ == "__main__":

    logger.enable_console(True)
    settings = Settings()

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) != 2:
        exit("the path to data bdlhes directory is missing")

    sitesdb = str(settings.value("General", "defaultSite"))

    conn = sqlite.connect(sitesdb)

    export_to_bdlhes(conn, sys.argv[1])
