# -*- coding: UTF-8 -*-
"""
load radionuclides table in sites database

USAGE

    python -m thyrsis.database.reset_radionuclides

    reset sites.sqlite database by :

        - reading radionuclides.csv file
        - redefining elements_chimiques table
        - determining new chemical idents in simulations table

    the expected radionuclides.csv file contains :

        "name",
        "period",
        "atomic_mass",
        "radioactive_decay",
        "specific_activity",
        "water_quality_limit",
        "atomic_number",
        "masse_number"
    
    the column separator is ";"

"""

import sys
import os
import shutil
import pandas as pd
from thyrsis.log import logger
from thyrsis.database import sqlite
from thyrsis.settings import Settings
from thyrsis.database.convert_radionuclides import convert_names

def reset_radionuclides(sitesdb):

    conn = sqlite.connect(sitesdb)
    cur = conn.cursor()

    print("reading radionuclides table")
    radionuclides_dir = os.path.dirname(os.path.abspath(sitesdb))
    radionuclides_file = os.path.join(radionuclides_dir, "radionuclides.csv")
    if not os.path.isfile(radionuclides_file):
        exit("radionuclides file not found in directory", radionuclides_dir)
    rows = pd.read_csv(radionuclides_file, sep=";")

    print("extracting old elements_chimiques table to get old chemical idents")
    old_chemical_dict = {x[0] : x[1] for x in cur.execute(
        "SELECT id, nom FROM elements_chimiques ORDER BY id").fetchall()}
    old_chemical_dict[None] = None

    print("saving missing chemical elements from simulations table")
    simulations_eid = sorted(list(set([x[0] for x in cur.execute(
        "SELECT eid FROM simulations WHERE eid IS NOT NULL").fetchall()])))
    simulations_enames = [convert_names(old_chemical_dict[eid]) for eid in simulations_eid]
    missing_enames = [x for x in simulations_enames if x not in [str(x) for x in rows.nom]]

    table = "elements_chimiques"
    missing_rows = cur.execute("SELECT * FROM {} WHERE nom IN ({})".format(
        table, ','.join(["'"+i+"'" for i in missing_enames]))).fetchall()
    missing_rows = [x[1:] for x in missing_rows]
    if "Tracer" not in [x[0] for x in missing_rows]:
        missing_rows.insert(0, ("Tracer", None, None, 0., 0., 0., None, None))

    print("adding",  len(missing_rows), "chemical elements to radionuclides table")
    rows = rows.append(pd.DataFrame(missing_rows, columns=rows.columns), ignore_index=True)

    rows["id"] = range(1, rows.shape[0]+1)

    print("deleting old elements_chimiques table")
    cur.execute("DROP TABLE IF EXISTS  elements_chimiques")

    print("creating new elements_chimiques table")
    cur.execute("""
        CREATE TABLE elements_chimiques(
            id INTEGER PRIMARY KEY,
            nom TEXT NOT NULL,
            periode REAL,
            masse_atomique REAL,
            decroissance_radioactive REAL,
            activite_specifique REAL NOT NULL DEFAULT 0,
            limite_qualite_eau REAL NOT NULL DEFAULT 0,
            numero_atomique INTEGER,
            nombre_de_masse INTEGER)""")

    print("updating elements_chimiques table with new radionuclides values")
    fields = [x[0] for x in cur.execute("SELECT name FROM pragma_table_info('{}')".format(table)).fetchall()]
    fields_str = ','.join(fields)
    rows = rows.reindex(columns=fields)
    cur.executemany("INSERT INTO {}({}) VALUES ({})".format(
        table, fields_str, ','.join(['?' for i in fields])),
        [tuple(x) for x in rows.astype("object").to_records(index=False)])

    new_chemical_dict = {x[1] : x[0] for x in cur.execute(
        "SELECT id, nom FROM elements_chimiques ORDER BY id").fetchall()}
    new_chemical_dict[None] = None

    for table in ("simulations", "mesures"):
        print("updating %s table to set new chemical idents"%(table))
        values = [(new_chemical_dict[convert_names(old_chemical_dict[eid])], eid) \
            for eid in set([x[0] for x in cur.execute(
                "SELECT eid FROM {} WHERE eid IS NOT NULL ORDER BY eid".format(table)).fetchall()])]
        print(values)
        cur.executemany("UPDATE {} SET eid = ? WHERE eid = ?".format(table), values)

    conn.commit()


if __name__ == "__main__":

    logger.enable_console(True)

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    #sitesdb = os.path.join(os.getenv("HOME"), ".thyrsis", "toto.sqlite")
    #shutil.copyfile(str(Settings().value("General", "defaultSite")), sitesdb)
    sitesdb = str(Settings().value("General", "defaultSite"))

    reset_radionuclides(sitesdb)

